package com.xc.utils.redis;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.xc.common.ServerResponse;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;

import org.slf4j.LoggerFactory;

import org.springframework.stereotype.Component;
import redis.clients.jedis.ShardedJedis;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

@Slf4j
@Component
public class RedisShardedPoolUtils {

    public static final int SAVE = 2;
    public static final int GET = 1;
    public static final int UPDATE = 0;
    public static final int DELETE = -1;

    // 注解缓存
    public static  <T> void saveJsonCache(String key,T t){
        ShardedJedis jedis = null;
        String result = null;
        String json = JSONObject.toJSONString(t);
        try {
            jedis = RedisShardedPool.getJedis();
            result = jedis.set(key, json);
        } catch (Exception e) {
            log.error("redis set key: {} value: {} error", new Object[]{key, json, e});
            RedisShardedPool.returnBrokenResouce(jedis);
        }
        RedisShardedPool.returnResouce(jedis);
    }

    public static Object getRedisJsonData(String prefix , MethodSignature signature){
        String s = get(prefix);
        //判断是否查询出数据，redis返回的数据是JSON格式的字符串
        if (null!=s){
            //获取连接点返回值类型，并将json格式字符串转成对应的类型返回
            Class returnType = signature.getReturnType();
            Gson gson = new Gson();
            return gson.fromJson(s, returnType);
        }
        return null;
    }

    /**判断是否存在key*/
    public boolean containKey(String key){
        boolean b;
        ShardedJedis jedis = null;
        try{
            jedis = RedisShardedPool.getJedis();
            b = jedis.exists(key);
            return b;
        }catch (Exception e){
            log.error("Redis server error:："+e.getMessage());
            RedisShardedPool.returnBrokenResouce(jedis);
            return false;
        }finally {
            RedisShardedPool.returnResouce(jedis);
        }
    }

    public static String set(String key, String value) {
        ShardedJedis jedis = null;
        String result = null;
        try {
            jedis = RedisShardedPool.getJedis();
            result = jedis.set(key, value);
        } catch (Exception e) {
            log.error("redis set key: {} value: {} error", new Object[]{key, value, e});
            RedisShardedPool.returnBrokenResouce(jedis);
            return result;
        }
        RedisShardedPool.returnResouce(jedis);
        return result;
    }

    public static String get(String key) {
        ShardedJedis jedis = null;
        String result = null;
        try {
            jedis = RedisShardedPool.getJedis();
            result = jedis.get(key);
        } catch (Exception e) {
            log.error("redis get key: {} error", key, e);
            RedisShardedPool.returnBrokenResouce(jedis);
            return result;
        }
        RedisShardedPool.returnResouce(jedis);
        return result;
    }

    public static String setEx(String key, String value, int exTime) {
        ShardedJedis jedis = null;
        String result = null;
        try {
            jedis = RedisShardedPool.getJedis();
            result = jedis.setex(key, exTime, value);
        } catch (Exception e) {
            log.error("redis setEx key: {} value: {}   error...", new Object[]{key, value, e});
            RedisShardedPool.returnBrokenResouce(jedis);
            return result;
        }
        RedisShardedPool.returnResouce(jedis);
        return result;
    }

    public static Long expire(String key, int exTime) {
        ShardedJedis jedis = null;
        Long result = null;
        try {
            jedis = RedisShardedPool.getJedis();
            result = jedis.expire(key, exTime);
        } catch (Exception e) {
            log.error("redis expire key: {}  error ", key, e);
            RedisShardedPool.returnBrokenResouce(jedis);
            return result;
        }
        RedisShardedPool.returnResouce(jedis);
        return result;
    }

    public static Long del(String key) {
        ShardedJedis jedis = null;
        Long result = null;
        try {
            jedis = RedisShardedPool.getJedis();
            result = jedis.del(key);
        } catch (Exception e) {
            log.error("redis del key: {} error ", key, e);
            RedisShardedPool.returnBrokenResouce(jedis);
            return result;
        }
        RedisShardedPool.returnResouce(jedis);
        return result;
    }

    public static <T> void setObject(String key,T t){
        ShardedJedis jedis = null;
        T result = null;
        try {
            jedis = RedisShardedPool.getJedis();
            jedis.set(key.getBytes(),serialize(t));
        } catch (Exception e) {
            log.error("redis set key: {} value: {} error", new Object[]{key, t, e});
            RedisShardedPool.returnBrokenResouce(jedis);
        }
        RedisShardedPool.returnResouce(jedis);
    }

    public static <T> T getObject(String key, T t){
        ShardedJedis jedis = null;
        T result = null;
        try {
            jedis = RedisShardedPool.getJedis();
            byte[] bytes = jedis.get(key.getBytes());
            if(null!=bytes && bytes.length>0){
                result = unserialize(bytes,t);
            }
        } catch (Exception e) {
            log.error("redis get key: {} value: {} error", new Object[]{key, t, e});
            RedisShardedPool.returnBrokenResouce(jedis);
            return result;
        }
        RedisShardedPool.returnResouce(jedis);
        return result;
    }

    //序列化
    public static <T> byte[] serialize(T object) {
        ObjectOutputStream oos = null;
        ByteArrayOutputStream baos = null;
        try {
            baos = new ByteArrayOutputStream();
            oos = new ObjectOutputStream(baos);
            oos.writeObject(object);
            byte[] bytes = baos.toByteArray();
            return bytes;
        } catch (Exception e) {
            log.error("序列化失败：{}",e);
        }
        return null;
    }

    //反序列化
    public static <T> T unserialize(byte[] bytes, T t) {
        ByteArrayInputStream bais = null;
        try {
            bais = new ByteArrayInputStream(bytes);
            ObjectInputStream ois = new ObjectInputStream(bais);
            return (T)ois.readObject();
        } catch (Exception e) {
            log.error("反序列化失败：{}",e);
        }
        return null;
    }

    public static void main(String[] args) {
        ShardedJedis jedis = RedisShardedPool.getJedis();
        System.out.println(get("key1"));
        System.out.println(get("key2"));
        System.out.println(get("key3"));
        System.out.println("redis shaded pool utils end ...");
    }

}